import {
  Body,
  Controller,
  Del,
  Get,
  Inject,
  Param,
  Post,
  Query,
  Config,
} from '@midwayjs/decorator';
import {
  BaseController,
  FrameworkErrorEnum,
  ServiceError,
} from 'free-midway-component';
import { UserService } from '../service/User.service';
import {
  CDTO_User,
  CVO_User,
  FVO_User,
  QDTO_User,
  QVO_User,
  UDTO_User,
  UVAO_User,
  LoginDTO,
  ResetPasswordDTO,
  LoginVO,
} from '../interface/User.interface';
import {
  ApiResponse,
  ApiTags,
  ApiQuery,
  ApiOperation,
} from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { JwtService } from '@midwayjs/jwt';
import { UserUtilService } from '../service/util/user.util';
@ApiTags(['用户'])
@Controller('/User')
export class UserController extends BaseController {
  @Inject()
  service: UserService;

  @Inject()
  jwtService: JwtService;

  @Config('jwt.secret')
  jwtSecret;

  @Config('jwt.expiresIn')
  jwtExpiresIn;

  @Inject()
  userUtil: UserUtilService;

  @Post('/login')
  @Validate()
  async login(@Body() body: LoginDTO) {
    const data = await this.service.findUserToLogin(body);
    let vo = new LoginVO(data);
    vo = JSON.parse(JSON.stringify(vo));
    const token = await this.jwtService.sign(vo, this.jwtSecret, {
      expiresIn: this.jwtExpiresIn,
    });
    return token;
  }

  @Post('/rp')
  @ApiOperation({ description: '修改密码' })
  async resetPassword(@Body() body: ResetPasswordDTO) {
    if (!body._id)
      throw new ServiceError('未找到用户信息', FrameworkErrorEnum.NOT_LOGIN);
    // 查询用户
    const user = await this.service.fetch(body._id);
    if (!user)
      throw new ServiceError('未找到用户', FrameworkErrorEnum.NOT_FOUND_DATA);
    // 修改密码
    await this.service.updateOne(body._id, { password: body.password });
    // 返回结果
    return 'ok';
  }

  @Post('/')
  @Validate()
  @ApiResponse({ type: CVO_User })
  async create(@Body() data: CDTO_User) {
    // 检查手机号和姓名
    await this.userUtil.checkPhoneAndPid(data);
    const dbData = await this.service.create(data);
    const result = new CVO_User(dbData);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_User })
  async query(
    @Query() filter: QDTO_User,
    @Query('skip') skip: number,
    @Query('limit') limit: number
  ) {
    const list = await this.service.query(filter, { skip, limit });
    const data = [];
    for (const i of list) {
      const newData = new QVO_User(i);
      data.push(newData);
    }
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/openid')
  @ApiResponse({ type: FVO_User })
  async wxLogin(@Query('openid') openid: string) {
    const data = await this.service.wxLogin(openid);
    let vo = new LoginVO(data);
    vo = JSON.parse(JSON.stringify(vo));
    const token = await this.jwtService.sign(vo, this.jwtSecret, {
      expiresIn: this.jwtExpiresIn,
    });
    return token;
  }

  @Get('/:id')
  @ApiResponse({ type: FVO_User })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FVO_User(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UVAO_User })
  async update(@Param('id') id: string, @Body() body: UDTO_User) {
    // 检查手机号和姓名
    await this.userUtil.checkUpdateCardAndPid(id, body);
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
